

#ifndef SBK_KCALENDARCORE_PYTHON_H
#define SBK_KCALENDARCORE_PYTHON_H

//workaround to access protected functions
#define protected public

#include <sbkpython.h>
#include <sbkmodule.h>
#include <sbkconverter.h>
// Module Includes
#include <pyside6_qtcore_python.h>
#include <pyside6_qtgui_python.h>

// Bound library includes
#include <alarm.h>
#include <attachment.h>
#include <attendee.h>
#include <calendar.h>
#include <calendarlistmodel.h>
#include <calendarplugin.h>
#include <calendarpluginloader.h>
#include <calfilter.h>
#include <calformat.h>
#include <calstorage.h>
#include <conference.h>
#include <customproperties.h>
#include <duration.h>
#include <event.h>
#include <exceptions.h>
#include <filestorage.h>
#include <freebusy.h>
#include <freebusycache.h>
#include <freebusyperiod.h>
#include <icalformat.h>
#include <incidence.h>
#include <incidencebase.h>
#include <journal.h>
#include <memorycalendar.h>
#include <occurrenceiterator.h>
#include <period.h>
#include <person.h>
#include <recurrence.h>
#include <recurrencerule.h>
#include <schedulemessage.h>
#include <sorting.h>
#include <todo.h>
#include <vcalformat.h>
#include <visitor.h>
// Conversion Includes - Primitive Types
#include <QAnyStringView>
#include <qbytearrayview.h>
#include <qchar.h>
#include <QtCore/qlatin1stringview.h>
#include <QString>
#include <QStringList>
#include <QStringView>
#include <qvariant.h>

// Conversion Includes - Container Types
#include <QHash>
#include <QList>
#include <QMap>
#include <QMultiHash>
#include <QMultiMap>
#include <QPair>
#include <QQueue>
#include <QSet>
#include <QStack>
#include <array>
#include <list>
#include <map>
#include <utility>
#include <unordered_map>
#include <vector>


// Type indices
enum : int {
    SBK_KCalendarCore_SortDirection_IDX                      = 59,
    SBK_KCalendarCore_EventSortField_IDX                     = 24,
    SBK_KCalendarCore_TodoSortField_IDX                      = 61,
    SBK_KCalendarCore_JournalSortField_IDX                   = 46,
    SBK_KCalendarCore_AccessMode_IDX                         = 1,
    SBK_KCalendarCore_iTIPMethod_IDX                         = 65,
    SBK_KCalendarCoreKCalendarCore_IDX                       = 0,
    SBK_KCalendarCore_Alarm_Type_IDX                         = 3,
    SBK_KCalendarCore_Alarm_IDX                              = 2,
    SBK_KCalendarCore_Attachment_IDX                         = 4,
    SBK_KCalendarCore_Attendee_PartStat_IDX                  = 7,
    SBK_KCalendarCore_Attendee_Role_IDX                      = 8,
    SBK_KCalendarCore_Attendee_CuType_IDX                    = 6,
    SBK_KCalendarCore_Attendee_IDX                           = 5,
    SBK_KCalendarCore_CalFilter_Criteria_IDX                 = 10,
    SBK_KCalendarCore_CalFilter_IDX                          = 9,
    SBK_KCalendarCore_CalFormat_IDX                          = 11,
    SBK_KCalendarCore_CalStorage_IDX                         = 12,
    SBK_KCalendarCore_Calendar_IDX                           = 13,
    SBK_KCalendarCore_CalendarListModel_Role_IDX             = 15,
    SBK_KCalendarCore_CalendarListModel_IDX                  = 14,
    SBK_KCalendarCore_CalendarPlugin_IDX                     = 16,
    SBK_KCalendarCore_CalendarPluginLoader_IDX               = 17,
    SBK_KCalendarCore_Conference_IDX                         = 18,
    SBK_KCalendarCore_CustomProperties_IDX                   = 19,
    SBK_KCalendarCore_Duration_Type_IDX                      = 21,
    SBK_KCalendarCore_Duration_IDX                           = 20,
    SBK_KCalendarCore_Event_Transparency_IDX                 = 23,
    SBK_KCalendarCore_Event_IDX                              = 22,
    SBK_KCalendarCoreKCalendarCore_Events_IDX                = 25,
    SBK_KCalendarCore_Exception_ErrorCode_IDX                = 27,
    SBK_KCalendarCore_Exception_IDX                          = 26,
    SBK_KCalendarCore_FileStorage_IDX                        = 28,
    SBK_KCalendarCore_FreeBusy_IDX                           = 29,
    SBK_KCalendarCore_FreeBusyCache_IDX                      = 30,
    SBK_KCalendarCore_FreeBusyPeriod_FreeBusyType_IDX        = 32,
    SBK_KCalendarCore_FreeBusyPeriod_IDX                     = 31,
    SBK_KCalendarCore_ICalFormat_IDX                         = 33,
    SBK_KCalendarCore_Incidence_Status_IDX                   = 37,
    SBK_KCalendarCore_Incidence_Secrecy_IDX                  = 36,
    SBK_KCalendarCore_Incidence_RelType_IDX                  = 35,
    SBK_KCalendarCore_Incidence_IDX                          = 34,
    SBK_KCalendarCore_IncidenceBase_IncidenceType_IDX        = 42,
    SBK_KCalendarCore_IncidenceBase_DateTimeRole_IDX         = 39,
    SBK_KCalendarCore_IncidenceBase_Field_IDX                = 40,
    SBK_KCalendarCore_IncidenceBase_VirtualHook_IDX          = 43,
    SBK_KCalendarCore_IncidenceBase_IDX                      = 38,
    SBK_KCalendarCore_IncidenceBase_IncidenceObserver_IDX    = 41,
    SBK_KCalendarCoreKCalendarCore_Incidences_IDX            = 44,
    SBK_KCalendarCore_Journal_IDX                            = 45,
    SBK_KCalendarCoreKCalendarCore_Journals_IDX              = 47,
    SBK_KCalendarCore_MemoryCalendar_IDX                     = 48,
    SBK_KCalendarCore_OccurrenceIterator_IDX                 = 49,
    SBK_KCalendarCore_Period_IDX                             = 50,
    SBK_KCalendarCore_Person_IDX                             = 51,
    SBK_KCalendarCore_Recurrence_IDX                         = 52,
    SBK_KCalendarCore_Recurrence_RecurrenceObserver_IDX      = 53,
    SBK_KCalendarCore_RecurrenceRule_PeriodType_IDX          = 55,
    SBK_KCalendarCore_RecurrenceRule_IDX                     = 54,
    SBK_KCalendarCore_RecurrenceRule_RuleObserver_IDX        = 56,
    SBK_KCalendarCore_ScheduleMessage_Status_IDX             = 58,
    SBK_KCalendarCore_ScheduleMessage_IDX                    = 57,
    SBK_KCalendarCore_Todo_IDX                               = 60,
    SBK_KCalendarCoreKCalendarCore_Todos_IDX                 = 62,
    SBK_KCalendarCore_VCalFormat_IDX                         = 63,
    SBK_KCalendarCore_Visitor_IDX                            = 64,
    SBK_KCalendarCore_IDX_COUNT                              = 66,
};

// This variable stores all Python types exported by this module.
extern Shiboken::Module::TypeInitStruct *SbkKCalendarCoreTypeStructs;

// This variable stores the Python module object exported by this module.
extern PyObject *SbkKCalendarCoreModuleObject;

// This variable stores all type converters exported by this module.
extern SbkConverter **SbkKCalendarCoreTypeConverters;

// Converter indices
enum [[deprecated]] : int {
    SBK_KCALENDARCORE_QLIST_INT_IDX                          = 0, // QList<int>
    SBK_KCALENDARCORE_QLIST_KCALENDARCORE_PERSON_IDX         = 1, // QList<KCalendarCore::Person>
    SBK_KCALENDARCORE_QLIST_QDATETIME_IDX                    = 2, // QList<QDateTime>
    SBK_KCALENDARCORE_QLIST_KCALENDARCORE_ATTACHMENT_IDX     = 3, // QList<KCalendarCore::Attachment>
    SBK_KCALENDARCORE_QLIST_KCALENDARCORE_CONFERENCE_IDX     = 4, // QList<KCalendarCore::Conference>
    SBK_KCALENDARCORE_QLIST_KCALENDARCORE_FREEBUSYPERIOD_IDX = 5, // QList<KCalendarCore::FreeBusyPeriod>
    SBK_KCALENDARCORE_QLIST_KCALENDARCORE_PERIOD_IDX         = 6, // QList<KCalendarCore::Period>
    SBK_KCALENDARCORE_QLIST_KCALENDARCORE_ATTENDEE_IDX       = 7, // QList<KCalendarCore::Attendee>
    SBK_KCALENDARCORE_QSET_KCALENDARCORE_INCIDENCEBASE_FIELD_IDX = 8, // QSet<KCalendarCore::IncidenceBase::Field>
    SBK_KCALENDARCORE_QMAP_QBYTEARRAY_QSTRING_IDX            = 9, // QMap<QByteArray,QString>
    SBK_KCALENDARCORE_QLIST_QTIME_IDX                        = 10, // QList<QTime>
    SBK_KCALENDARCORE_QLIST_QDATE_IDX                        = 11, // QList<QDate>
    SBK_KCALENDARCORE_QLIST_KCALENDARCORE_RECURRENCERULEPTR_IDX = 12, // QList<KCalendarCore::RecurrenceRule*>
    SBK_KCALENDARCORE_QMAP_INT_QVARIANT_IDX                  = 13, // QMap<int,QVariant>
    SBK_KCALENDARCORE_QLIST_QMODELINDEX_IDX                  = 14, // QList<QModelIndex>
    SBK_KCALENDARCORE_QHASH_INT_QBYTEARRAY_IDX               = 15, // QHash<int,QByteArray>
    SBK_KCALENDARCORE_QLIST_QVARIANT_IDX                     = 16, // QList<QVariant>
    SBK_KCALENDARCORE_QLIST_QSTRING_IDX                      = 17, // QList<QString>
    SBK_KCALENDARCORE_QMAP_QSTRING_QVARIANT_IDX              = 18, // QMap<QString,QVariant>
    SBK_KCALENDARCORE_CONVERTERS_IDX_COUNT                   = 19,
};

// Converter indices
enum : int {
    SBK_KCalendarCore_QList_int_IDX                          = 0, // QList<int>
    SBK_KCalendarCore_QList_KCalendarCore_Person_IDX         = 1, // QList<KCalendarCore::Person>
    SBK_KCalendarCore_QList_QDateTime_IDX                    = 2, // QList<QDateTime>
    SBK_KCalendarCore_QList_KCalendarCore_Attachment_IDX     = 3, // QList<KCalendarCore::Attachment>
    SBK_KCalendarCore_QList_KCalendarCore_Conference_IDX     = 4, // QList<KCalendarCore::Conference>
    SBK_KCalendarCore_QList_KCalendarCore_FreeBusyPeriod_IDX = 5, // QList<KCalendarCore::FreeBusyPeriod>
    SBK_KCalendarCore_QList_KCalendarCore_Period_IDX         = 6, // QList<KCalendarCore::Period>
    SBK_KCalendarCore_QList_KCalendarCore_Attendee_IDX       = 7, // QList<KCalendarCore::Attendee>
    SBK_KCalendarCore_QSet_KCalendarCore_IncidenceBase_Field_IDX = 8, // QSet<KCalendarCore::IncidenceBase::Field>
    SBK_KCalendarCore_QMap_QByteArray_QString_IDX            = 9, // QMap<QByteArray,QString>
    SBK_KCalendarCore_QList_QTime_IDX                        = 10, // QList<QTime>
    SBK_KCalendarCore_QList_QDate_IDX                        = 11, // QList<QDate>
    SBK_KCalendarCore_QList_KCalendarCore_RecurrenceRulePTR_IDX = 12, // QList<KCalendarCore::RecurrenceRule*>
    SBK_KCalendarCore_QMap_int_QVariant_IDX                  = 13, // QMap<int,QVariant>
    SBK_KCalendarCore_QList_QModelIndex_IDX                  = 14, // QList<QModelIndex>
    SBK_KCalendarCore_QHash_int_QByteArray_IDX               = 15, // QHash<int,QByteArray>
    SBK_KCalendarCore_QList_QVariant_IDX                     = 16, // QList<QVariant>
    SBK_KCalendarCore_QList_QString_IDX                      = 17, // QList<QString>
    SBK_KCalendarCore_QMap_QString_QVariant_IDX              = 18, // QMap<QString,QVariant>
    SBK_KCalendarCore_CONVERTERS_IDX_COUNT                   = 19,
};
// Macros for type check

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
namespace Shiboken
{

// PyType functions, to get the PyObjectType for a type T
template<> inline PyTypeObject *SbkType< ::KCalendarCore::SortDirection >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_SortDirection_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::EventSortField >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_EventSortField_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::TodoSortField >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_TodoSortField_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::JournalSortField >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_JournalSortField_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::AccessMode >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_AccessMode_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::iTIPMethod >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_iTIPMethod_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Alarm::Type >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Alarm_Type_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Alarm >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Alarm_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Attachment >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Attachment_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Attendee::PartStat >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Attendee_PartStat_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Attendee::Role >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Attendee_Role_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Attendee::CuType >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Attendee_CuType_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Attendee >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Attendee_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::CalFilter::Criteria >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_CalFilter_Criteria_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::CalFilter >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_CalFilter_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::CalFormat >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_CalFormat_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::CalStorage >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_CalStorage_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Calendar >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Calendar_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::CalendarListModel::Role >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_CalendarListModel_Role_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::CalendarListModel >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_CalendarListModel_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::CalendarPlugin >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_CalendarPlugin_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::CalendarPluginLoader >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_CalendarPluginLoader_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Conference >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Conference_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::CustomProperties >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_CustomProperties_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Duration::Type >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Duration_Type_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Duration >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Duration_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Event::Transparency >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Event_Transparency_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Event >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Event_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Exception::ErrorCode >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Exception_ErrorCode_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Exception >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Exception_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::FileStorage >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_FileStorage_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::FreeBusy >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_FreeBusy_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::FreeBusyCache >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_FreeBusyCache_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::FreeBusyPeriod::FreeBusyType >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_FreeBusyPeriod_FreeBusyType_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::FreeBusyPeriod >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_FreeBusyPeriod_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::ICalFormat >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_ICalFormat_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Incidence::Status >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Incidence_Status_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Incidence::Secrecy >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Incidence_Secrecy_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Incidence::RelType >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Incidence_RelType_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Incidence >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Incidence_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::IncidenceBase::IncidenceType >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_IncidenceBase_IncidenceType_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::IncidenceBase::DateTimeRole >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_IncidenceBase_DateTimeRole_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::IncidenceBase::Field >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_IncidenceBase_Field_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::IncidenceBase::VirtualHook >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_IncidenceBase_VirtualHook_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::IncidenceBase >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_IncidenceBase_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::IncidenceBase::IncidenceObserver >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_IncidenceBase_IncidenceObserver_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Journal >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Journal_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::MemoryCalendar >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_MemoryCalendar_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::OccurrenceIterator >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_OccurrenceIterator_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Period >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Period_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Person >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Person_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Recurrence >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Recurrence_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Recurrence::RecurrenceObserver >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Recurrence_RecurrenceObserver_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::RecurrenceRule::PeriodType >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_RecurrenceRule_PeriodType_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::RecurrenceRule >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_RecurrenceRule_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::RecurrenceRule::RuleObserver >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_RecurrenceRule_RuleObserver_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::ScheduleMessage::Status >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_ScheduleMessage_Status_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::ScheduleMessage >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_ScheduleMessage_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Todo >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Todo_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::VCalFormat >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_VCalFormat_IDX]); }
template<> inline PyTypeObject *SbkType< ::KCalendarCore::Visitor >() { return Shiboken::Module::get(SbkKCalendarCoreTypeStructs[SBK_KCalendarCore_Visitor_IDX]); }

} // namespace Shiboken

QT_WARNING_POP
#endif // SBK_KCALENDARCORE_PYTHON_H

